"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.aggregate = exports.AggregateMethod = void 0;
const lodash_1 = require("lodash");
const luxon_1 = require("luxon");
const bigNumber_1 = require("./bigNumber");
var AggregateMethod;
(function (AggregateMethod) {
    AggregateMethod["Avg"] = "avg";
    AggregateMethod["Sum"] = "sum";
})(AggregateMethod = exports.AggregateMethod || (exports.AggregateMethod = {}));
const buildAggregateFrames = ({ data, window, start, end }) => {
    if (data.length === 0 && (!start || !end)) {
        return [];
    }
    const intervalStart = start ?? lodash_1.first(data).time;
    const intervalEnd = end ?? lodash_1.last(data)?.time ?? new Date();
    const intervalStartRounded = new Date(window.roundDate(intervalStart));
    const intervals = luxon_1.Interval.fromDateTimes(intervalStartRounded, intervalEnd).splitBy(window.toLuxonDuration());
    return intervals.map((interval, i) => ({
        start: i === 0 ? intervalStart : interval.start.toJSDate(),
        end: interval.end.toJSDate()
    }));
};
const offsetByTimezone = (command) => {
    const { start, end, data, ...rest } = command;
    if (command.timezoneOffset === 0) {
        return command;
    }
    const milisOffset = command.timezoneOffset * 60 * 1000;
    return {
        start: start ? new Date(start.getTime() - milisOffset) : undefined,
        end: end ? new Date(end.getTime() - milisOffset) : undefined,
        data: data.map((d) => ({
            ...d,
            time: new Date(d.time.getTime() - milisOffset)
        })),
        ...rest
    };
};
const aggregate = ({ data, method, window, start, end, timezoneOffset }) => {
    const aggregateFunction = method === 'sum' ? bigNumber_1.bigNumSum : bigNumber_1.bigNumAvg;
    const sortedData = lodash_1.orderBy(data, [(e) => e.time], 'asc');
    const offsettedCommand = offsetByTimezone({
        data: sortedData,
        method,
        window,
        start,
        end,
        timezoneOffset
    });
    const intervals = buildAggregateFrames(offsettedCommand);
    return intervals
        .map((interval, intervalN) => {
        const resultsInInterval = offsettedCommand.data.filter((r, dataN) => (r.time > interval.start && r.time <= interval.end) ||
            (start === undefined && intervalN === 0 && dataN === 0));
        return {
            start: interval.start,
            end: interval.end,
            results: resultsInInterval.map((r) => r.value)
        };
    })
        .map((e) => ({
        start: e.start,
        end: e.end,
        value: aggregateFunction(e.results)
    }));
};
exports.aggregate = aggregate;
//# sourceMappingURL=aggregate.js.map